package com.yourdomain.agent;

import net.bytebuddy.agent.builder.AgentBuilder;
import net.bytebuddy.asm.Advice;
import net.bytebuddy.matcher.ElementMatchers;

import java.io.FileWriter;
import java.io.PrintWriter;
import java.lang.instrument.Instrumentation;
import java.lang.reflect.Field;
import java.time.LocalDateTime;

public class CheckrpAgent {

    public static void premain(String agentArgs, Instrumentation inst) {
        System.out.println("[CheckrpAgent] Agent loaded: Logging sent packets to sent_packets.txt");

        new AgentBuilder.Default()
                .type(ElementMatchers.nameEndsWith("ModLoadProcedure"))
                .transform((builder, typeDescription, classLoader, module, protectionDomain) ->
                        builder.method(ElementMatchers.named("sendToServer"))
                                .intercept(Advice.to(SendPacketLogger.class))
                )
                .installOn(inst);

    }

    public static class SendPacketLogger {

        @Advice.OnMethodEnter
        public static void onEnter(@Advice.Argument(0) Object packet) {
            try (PrintWriter out = new PrintWriter(new FileWriter("sent_packets.txt", true))) {
                out.println("==== Packet Sent ====");
                out.println("Time: " + LocalDateTime.now());
                out.println("Class: " + packet.getClass().getName());
                for (Field field : packet.getClass().getDeclaredFields()) {
                    field.setAccessible(true);
                    try {
                        Object value = field.get(packet);
                        out.printf("  %s = %s%n", field.getName(), value);
                    } catch (IllegalAccessException e) {
                        out.printf("  %s = <unreadable>%n", field.getName());
                    }
                }
                out.println("=====================\n");
            } catch (Exception e) {
                System.err.println("[CheckrpAgent] Failed to write packet info: " + e.getMessage());
            }
        }
    }
}
